// MyScript.h : Declaration of the CMyScript

#ifndef __MYSCRIPT_H_
#define __MYSCRIPT_H_


//#include "stdafx.h"

#include "resource.h"       // main symbols

/* Interface implemented

  Create simple ATL object
  in classview rightclick "Implement Interface .."
  select ContactCOM.tbl
  select Script interface 

*/
#import "ContactCOM.tlb" raw_interfaces_only, raw_native_types, no_namespace, named_guids 

// safe release of a COM ptr 
#define RELEASE(x) if (x) { x->Release(); x = NULL; }


// includes we need for the demo
#include <math.h>

 
/////////////////////////////////////////////////////////////////////////////
/*! CMyScript

  An ATL class implementing
  the Script Interface from ContactCOM.tlb


*/
class ATL_NO_VTABLE CMyScript : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CMyScript, &CLSID_MyScript>,
	public IDispatchImpl<IMyScript, &IID_IMyScript, &LIBID_NATIVESCRIPTDEMOLib>,
	public IDispatchImpl<Script, &IID_Script, &LIBID_ContactComLib>
{
public:

	CMyScript() : fieldsAreOk(false),
		h(NULL),dist(NULL),xmax(0),zmax(0)
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_MYSCRIPT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CMyScript)
	COM_INTERFACE_ENTRY(IMyScript)
//DEL 	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY2(IDispatch, IMyScript)
	COM_INTERFACE_ENTRY(Script)
END_COM_MAP()


private :
	// private script state

	// pointer to CC3D script node containing this object 

	CComPtr<Node> script;

	bool fieldsAreOk;

	// fields we are interested in 
	//field SFInt32 xDimension 
	CComQIPtr<EventOutSFInt32> xDimension;

	//field SFInt32 zDimension
	CComQIPtr<EventOutSFInt32> zDimension;

	//field SFFLoat r
	// queuried on demand

	//eventOut MFFloat heights
	CComQIPtr<EventInMFFloat> heights;


	// global state
	// simply shadowed here 
	int xmax,zmax;
	float r;

	float *h;
	float *dist;


// IMyScript
public:
// Script


	// script instance is created
	// we get the pointer to the owning Script node
	// inorder to get access to fields (and browser object)

	STDMETHOD(setContainer)(Node * container)
	{
		script = container;
		if (script) {

			// get any field pointers  needed 

			Field *tmp=NULL;

			// modify here : 
			script->getField( L"xDimension",&tmp);
			xDimension = tmp; RELEASE(tmp); 

			script->getField( L"zDimension",&tmp);
			zDimension = tmp; RELEASE(tmp); 


			script->getField( L"heights",&tmp);
			heights = tmp; RELEASE(tmp); 

			fieldsAreOk= (xDimension) && (zDimension) && (heights);

		} else {
			// important zero pointers to clean up cyclic references
			xDimension.Release();
			zDimension.Release();
			heights.Release();

			fieldsAreOk = false;

		}
		return S_OK;
	}

	// optional method to get data from url
	STDMETHOD(loadScriptObject)(BSTR urlData)
	{
		// 
		return E_NOTIMPL;
	}

	// the initialize method
	STDMETHOD(initialize)()
	{

		// protect against bad , missing fields 
		if (!fieldsAreOk)
			return E_POINTER;

		xDimension->getValue(&xmax);
		zDimension->getValue(&zmax);

		r=1.0f;

		// just a quick test for the optional field r
		{
			Field *tmp=NULL;

			script->getField( L"r",&tmp);
			
			CComQIPtr<EventOutSFFloat> rtmp(tmp);
			RELEASE(tmp); 
			if (rtmp)
				rtmp->getValue(&r);
		}



		h = new float [xmax*zmax];
		dist = new float [xmax*zmax];

		if (!h || !dist)
			return E_POINTER;
					
		float xCenter = (xmax * 1)/2.0f; 
		float zCenter = (zmax * 1)/2.0f; 

		for (int i = 0; i < zmax; i++)
		{
			for (int j = 0; j < xmax; j++)
			{
				 dist[i*xmax+j] = r* (float) sqrt((zCenter - i)*(zCenter - i) + (xCenter - j)*(xCenter - j));
			}
		}		

		

		return S_OK;

	}

	// the shutdown method
	STDMETHOD(shutdown)()
	{
		// cleanup stuff done in initialize 
		delete dist; dist = NULL;

		delete h; h = NULL;

		return S_OK;
	}
	
	// the processEvent method
	STDMETHOD(processEvent)(
		BSTR name, // name of eventIn function
		INT eventId, // ID, for speed, numbered by definition order (+3) for built-in fields
		EventOut * value,	// the value 
		DOUBLE timeStamp
		)
	{

		if (!fieldsAreOk)
			return E_POINTER;
		
		//TRACE("Got event %d \n",eventId);


		// assume the fraction eventIn function
		CComQIPtr<EventOutSFFloat> fraction(value);

		if (!fraction)
			return E_POINTER;

		float fvalue;
		fraction->getValue(&fvalue);


		if (!h || !dist)
			return E_POINTER;

		// now do something 

		int ioffset;
		float t=(6.28f*fvalue);
		
		for (int i = 0; i < zmax; i++)
		{	ioffset = i*xmax;
			for (int j = 0; j < xmax; j++)
			{	
				 h[ioffset+j] = (float) sin( (t - dist[ioffset+j]));
			}
		}

		// set the eventOut, triggering a ROUTE 
		heights->setValue(xmax*zmax,h);



		return S_OK;
	}


	STDMETHOD(eventsProcessed)()
	{
		// could do the work here, depending on whicht event has been changed
		return E_NOTIMPL;
	}
};

#endif //__MYSCRIPT_H_
